#include <stdio.h>
#include <malloc.h>
#include "String.h"

// Constructors

// Std-Constructor
String :: String()
{
	m_len = 0;
	m_p = new char[0];
}

// Constructor mit bergabe einer nullterminierten Zeichenkette
String :: String(const char *s)
{
	m_len = 0;
	while (s[m_len] != '\0')
		m_len++;
	m_p = new char[m_len];
	for(int i=0; i<m_len; i++)
		m_p[i] = s[i];
}

// Copy-Constructor
String :: String(const String &s)
{
	m_len = s.m_len;
	m_p = new char[m_len];
	for (int i=0; i<m_len; i++)
		m_p[i] = s.m_p[i];
}

// Destructor
String :: ~String()
{
	delete[] m_p;
}

int String :: Length()
{
	return this->m_len;
}

// Insert Zeichenkette
bool String :: Insert(const char *what, int where)
{
	if (where > m_len || where < 0)
		return false;

	int i, ii;

	// Lnge der einzufgenden Zeichenkette merken
	int lenWhat = strLen(what);

	// Alte Zeichenkette merken
	char *tmp;
	tmp = new char[m_len];
	for (i=0; i<m_len; i++)
		tmp[i] = m_p[i];

	// Lnge des Objekts um Lnge der Zeichenkette erweitern
	m_len += lenWhat;

	// Speicherbereich neu dimensionieren
	delete[] m_p;
	m_p = new char[m_len];

	// Beginn des "alten" Objektes an das neue anfgen
	for(i=0, ii=0; i<where; i++, ii++)
		m_p[i] = tmp[ii];
	
	// Zeichenkette in Mitte des Objekts schreiben
	for(i=where, ii=0; i<=where+lenWhat; i++, ii++)
		m_p[i] = what[ii];

	// Ende des "alten" Objekts an "neues" Objekt anfgen
	for(i=where+lenWhat, ii=where; i<m_len; i++, ii++)
		m_p[i] = tmp[ii];

	delete[] tmp;
	
	return true;
}

// Insert Objekt
bool String :: Insert(const String &what, int where)
{
	if (where > m_len || where < 0)
		return false;

	int i, ii;

	// Alte Zeichenkette merken
	char *tmp;
	tmp = new char[m_len];
	for (i=0; i<m_len; i++)
		tmp[i] = m_p[i];

	// Lnge des Objekts um Lnge der Zeichenkette erweitern
	m_len += what.m_len;

	// Speicherbereich neu dimensionieren
	delete[] m_p;
	m_p = new char[m_len];

	// Beginn des "alten" Objektes an das neue anfgen
	for(i=0, ii=0; i<where; i++, ii++)
		m_p[i] = tmp[ii];
	
	// Zeichenkette in Mitte des Objekts schreiben
	for(i=where, ii=0; i<=where+what.m_len; i++, ii++)
		m_p[i] = what.m_p[ii];

	// Ende des "alten" Objekts an "neues" Objekt anfgen
	for(i=where+what.m_len, ii=where; i<m_len; i++, ii++)
		m_p[i] = tmp[ii];

	delete[] tmp;
	
	return true;
}

// Insert Zeichen
bool String :: Insert(const char &what, int where)
{
	if (where > m_len || where < 0)
		return false;

	int i, ii;

	// Alte Zeichenkette merken
	char *tmp;
	tmp = new char[m_len];
	for (i=0; i<m_len; i++)
		tmp[i] = m_p[i];

	// Lnge des Objekts um 1 erweitern
	m_len += 1;

	// Speicherberich neu dimensionieren
	delete[] m_p;
	m_p = new char[m_len];

	// Beginn des "alten" Objektes an das neue anfgen
	for(i=0, ii=0; i<where; i++, ii++)
		m_p[i] = tmp[ii];
	
	// Zeichenkette in Mitte des Objekts schreiben
	m_p[where] = what;

	// Ende des "alten" Objekts an "neues" Objekt anfgen
	for(i=where+1, ii=where; i<m_len; i++, ii++)
		m_p[i] = tmp[ii];

	delete[] tmp;

	return true;
}

bool String :: Remove(int where, int len)
{
	int i, ii;
	char *tmp;

	// Alte Zeichenkette merken	
	tmp = new char[m_len];
	for (i=0; i<m_len; i++)
		tmp[i] = m_p[i];

	// Lnge verkleinern
	m_len -= len;

	// Speicherbereich neu dimensionieren
	delete m_p;
	m_p = new char[m_len];

	// Neue Zeichenkette fllen
	for(i=0; i<where; i++)
		m_p[i] = tmp[i];

	for(i=where, ii=0; i<m_len; i++, ii++)
		m_p[i] = tmp[where+len+ii];

	delete[] tmp;

	return true;
}

// Compare without Case-sensivity
bool String :: CmpWithoutCaseSens(const String &what)
{
	if(this->m_len != what.m_len) return false; // Lnge ungleich?

	for(int i=0; i<this->m_len; i++) // Elemente ungleich?
	{
		// Vergleich der ASCII-Werte: A[65] == A[65] || A[65] == a[97] || a[97] == A[65]
		
		// Wenn this->Char[i] == [A-Z]
		if ((int)this->m_p[i] > 64 && (int)this->m_p[i] < 91)
		{
			if (((int)this->m_p[i] != (int)what.m_p[i]) && ((int)this->m_p[i] != (int)what.m_p[i] - 32))
				return false;
		}// Wenn this->Char[i] == [a-z]
		else if ((int)this->m_p[i] > 96 && (int)this->m_p[i] < 123)
		{
			if (((int)this->m_p[i] != (int)what.m_p[i]) && ((int)this->m_p[i] != (int)what.m_p[i] + 32))
				return false;
		}// Wenn this->Char[i] != [A-Z, a-z] mu Gleichheit gelten
		else
            if((int)this->m_p[i] != (int)what.m_p[i])
				return false;
	}

	return true; // Strings gleich.
}

// Delete Whitespaces
void String :: DeleteWhitespaces()
{
	// Prfung der Lnge
	if (this->m_len <= 0)
		return;

	// Prfung der ersten und letzten Zeichen von auen nach innen
	int i, ii, tmpStart, tmpEnd;
	i = 0;
	ii = m_len-1;
	while (m_p[i] == ' ' || m_p[i] == '\t' || m_p[ii] == ' ' || m_p[ii] == '\t')
	{
		if (m_p[i] == ' ' || m_p[i] == '\t') i++;
		if (m_p[ii] == ' ' || m_p[ii] == '\t') ii--;
	}

	tmpStart = i;
	tmpEnd = ii+1;

	// Zuerst Ende manipulieren: Ende faul
	if (ii != m_len-1)
	{
		// Alte Zeichenkette ohne Whitespaces am Ende merken
		char *tmp = new char[tmpEnd];
		for(ii=tmpEnd-1; ii>=0; ii--)
			tmp[ii] = m_p[ii];

		// Alten Pointer neu dimensionieren, gekrzte Kette hineinkopieren
		delete[] m_p;
		m_p = tmp;

		// Lnge neu definieren
		m_len = tmpEnd;
	}

	// Anfang faul
	if (i != 0)
	{
		// Alte Zeichenkette ohne Whitespaces am Anfang merken
		char *tmp = new char[m_len-i];
		for(i=0; i<m_len; i++)
			tmp[i] = m_p[tmpStart+i];

		// Alten Pointer neu dimensionieren, gekrzte Kette hineinkopieren
		delete[] m_p;
		m_p = tmp;

		// Lnge neu definieren
		m_len -= tmpStart;
	}	

	return;
}

//int Find(const String &what)
//{
//
//}

String ::operator int ()
{
	return m_len;
}

String & String :: operator=(const String &what)
{
	if(m_p == what.m_p)
		return *this;

	// deep copy
	m_len = what.m_len;
	delete m_p;
	m_p = new char[what.m_len];
	for (int i=0; i<what.m_len; i++)
		m_p[i] = what.m_p[i];

	return *this;
}

bool String :: operator==(const String &what) const
{
	if(this->m_len != what.m_len) return false; // Lnge ungleich?
	for(int i=0; i<this->m_len; i++) // Elemente ungleich?
		if (this->m_p[i] != what.m_p[i]) return false;
	return true; // Strings gleich.
}

char & String :: operator[](int index)
{
	static char cDummy = '\0';
	if(index < 0 || index >= m_len)
		return cDummy;
	return m_p[index];
}

void String :: Print()
{
	for(int i=0; i<this->m_len; i++)
		printf("%c", this->m_p[i]);
}

int String :: strLen(const char *s)
{
	int i = -1;
	do
	{
		i++;
	}while(s[i] != '\0');
	return i;
}